// SliderChild.cpp : implementation file
//

#include "stdafx.h"
#include "Slider.h"
#include "SliderChild.h"
#include "lsapi.h"
#include "DIBSectionLite.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSliderChild

#define MOUSE_OVER_TIMER_ID		2
#define PANEL_DEFAULT			20000		// Used to determine if the user is putting 
											// a custom panel location

#define PANEL_DEFAULT_STEP		30000		// Makes image just appear no animiation


IMPLEMENT_DYNCREATE(CSliderChild, CWnd);

CSliderChild::CSliderChild()
{
	m_bExpanded = FALSE;

	m_OrigRect.SetRectEmpty();
	m_bDrawTransparent = TRUE;

	m_hRgn = NULL;

	m_cTopBitmapPath	= _T("");
	m_cBottomBitmapPath	= _T("");
	m_cMiddleBitmapPath	= _T("");
	m_MouseOverSound	= _T("");
	m_PressSound		= _T("");

	m_cPath				= _T("");
	m_cExclusionList	= _T("");
	m_WildCardSearch	= _T("*.lnk");

	m_bPrintText		= TRUE;
	m_bPrintIcon		= TRUE;
	m_bUseSmallIcons	= TRUE;
	m_bUseToolTip		= TRUE;

	m_nIconXAdj			= 0;
	m_nIconYAdj			= 0;

	m_nTextXAdj			= 0;
	m_nTextYAdj			= 0;

	m_cFontText			= _T("Small Fonts");
	m_nFontHeight		= 10;

	m_clrFontColor		= RGB(0,0,0);
	m_clrFontHighliteColor = RGB(255,255,255);
	
	m_nSlideDirection	= 1;
	m_nStep				= 30;

	m_bMouseTracking	= FALSE;

	m_nFadeStep			= 0;
	m_nFadeDelay		= 0;

	m_nCurBtn			= 0;
	m_nOldBtn			= 0;
	m_nSliderNumber		= 0;

	m_nTransparency		= 0;	// No transparency
	m_nTopBitmapHeight	= 0;

}

CSliderChild::~CSliderChild()
{
	// Remove all the buttons
	for(int i = m_colSliderButtons.GetUpperBound(); i >= 0; i--)
	{
		delete m_colSliderButtons[i];
	}

	m_colSliderButtons.RemoveAll();
}


BEGIN_MESSAGE_MAP(CSliderChild, CWnd)
	//{{AFX_MSG_MAP(CSliderChild)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_KILLFOCUS()
	ON_WM_SETCURSOR()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
END_MESSAGE_MAP()

void CopyBitmapToClipboardCM( const CWnd *pWnd, CBitmap& bitmap, CPalette* pPal )
{
	::OpenClipboard(pWnd->GetSafeHwnd());
	::EmptyClipboard() ;
	if( pPal )
		::SetClipboardData (CF_PALETTE, pPal->GetSafeHandle() ) ;
	::SetClipboardData (CF_BITMAP, bitmap.GetSafeHandle() ) ;
	::CloseClipboard () ;
	bitmap.Detach();
	if( pPal )
		pPal->Detach();
}

/////////////////////////////////////////////////////////////////////////////
// CSliderChild message handlers
BOOL CSliderChild::PreTranslateMessage(MSG* pMsg) 
{
	return CWnd::PreTranslateMessage(pMsg);
}

BOOL CSliderChild::CreateChildSlider(int nSliderNbr, HWND hParent)
{
	if(!IsWindow(hParent))
		return FALSE;

	if(!LoadSettings(nSliderNbr))
		return FALSE;

	// Calc out rect position
	CalcPosition(hParent);

	CString txtClassName = AfxRegisterWndClass(CS_DBLCLKS | CS_SAVEBITS);		

	BOOL rtn = CreateEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST, txtClassName, "LsLnkMenuChild", WS_POPUP | WS_VISIBLE, 
			m_OrigRect.left, m_OrigRect.top, 0, 0, hParent, 0);

	if(!rtn)
		return FALSE;
	
	// Make sure window stays put
	SetWindowLong(m_hWnd, GWL_USERDATA, magicDWord);

	CWindowDC dc(NULL);

	// Create the font stuff
	if(m_Font.CreatePointFont(10 * m_nFontHeight, m_cFontText, &dc))
		SetFont(&m_Font, FALSE);

	if(m_bUseToolTip)
		m_ToolTip.Create(this, TRUE);

	m_nOrientation = GetProfileIntValue(nSliderNbr, "Orientation", 1);
	m_bDynamic = GetProfileIntValue(nSliderNbr, "Dynamic", 0);

	if(!GetProfileIntValue(nSliderNbr, "LnkFilesOnly", 1))
	{
		// Change the wildcard
		m_WildCardSearch = "*.*";
	}

	if(!m_bDynamic)
	{
		if(1 == m_nOrientation)
			AssembleVerticalBitmap();
		else
			AssembleHorizontalBitmap();
	}

	return rtn;
}

void CSliderChild::CalcPosition(HWND hParent)
{
	// Function will calculate the m_OrigRect parameters
	CRect ParentRect;

	::GetWindowRect(hParent, ParentRect);

	CPoint pt = ConvertToScreenPosition(GetProfileIntValue(m_nSliderNumber, "PanelXPos", PANEL_DEFAULT),
										GetProfileIntValue(m_nSliderNumber, "PanelYPos", PANEL_DEFAULT));

	m_OrigRect.left		= pt.x;
	m_OrigRect.top		= pt.y;

	if(m_OrigRect.left == PANEL_DEFAULT || m_OrigRect.top == PANEL_DEFAULT)
	{
		// Calculate it
		switch(m_nSlideDirection)
		{
			case 1:
			case 5: 
			case 7:/* Slide Down variations*/
				m_OrigRect.left = ParentRect.left;
				m_OrigRect.top  = ParentRect.bottom;
				break;

			case 4:	/* Slide Right */
				m_OrigRect.left = ParentRect.right;
				m_OrigRect.top  = ParentRect.top;
				break;

			default:
				m_OrigRect.left = ParentRect.left;
				m_OrigRect.top  = ParentRect.top;
				break;
		}
	}
}

BOOL CSliderChild::LoadSettings(int nSliderNbr)
{
	CString cTemp	= _T("");

	// Load all the settings
	m_nSliderNumber = nSliderNbr;

	m_cTopBitmapPath	= GetLSLnkImagePath(nSliderNbr,	"PanelTopBitmap");
	m_cBottomBitmapPath = GetLSLnkImagePath(nSliderNbr, "PanelBottomBitmap");

	// This is required
	m_cMiddleBitmapPath = GetLSLnkImagePath(nSliderNbr, "PanelMiddleBitmap");

	if(!m_cMiddleBitmapPath.GetLength())
	{
		ErrorLog("No PanelMiddleBitmap entry in INI file!");
		return FALSE;
	}

	m_cPath = GetProfileStringValue(nSliderNbr, "LnkPath");

	// This is required
	if(!m_cPath.GetLength())
	{
		ErrorLog("No LnkPath entry in the INI file");
		return FALSE;
	}

	// See if we need to append a backslash
	if(m_cPath.Right(1) != '\\')
		m_cPath += '\\';

	m_cExclusionList	= GetProfileStringValue(nSliderNbr, "ExclusionList");

	m_cExclusionList.MakeUpper();

	m_nSlideDirection	= GetProfileIntValue(nSliderNbr, "SlideDirection", 1);
	m_nStep				= GetProfileIntValue(nSliderNbr, "SlideSpeed", 0);

	if(m_nStep == 0)
		m_nStep = PANEL_DEFAULT_STEP;

	m_bPrintText		= (BOOL)GetProfileIntValue(nSliderNbr, "ShowText", 1);
	m_bPrintIcon		= (BOOL)GetProfileIntValue(nSliderNbr, "ShowIcon", 1);
	m_bUseSmallIcons	= (BOOL)GetProfileIntValue(nSliderNbr, "UseSmallIcon", 1);

	m_nIconXAdj			= GetProfileIntValue(nSliderNbr, "IconXPos", 0);
	m_nIconYAdj			= GetProfileIntValue(nSliderNbr, "IconYPos", 0);

	m_nTextXAdj			= GetProfileIntValue(nSliderNbr, "TextXPos", 0);
	m_nTextYAdj			= GetProfileIntValue(nSliderNbr, "TextYPos", 0);

	m_nFontHeight		= GetProfileIntValue(nSliderNbr, "FontHeight", 10);

	m_cFontText			= GetProfileStringValue(nSliderNbr, "FontName", "Small Fonts");
	
	m_MouseOverSound	= GetProfileStringValue(nSliderNbr, "PanelMouseOverSound", "");
	m_PressSound		= GetProfileStringValue(nSliderNbr, "PanelPressSound"	 , "");

	int nR				= GetProfileIntValue(nSliderNbr, "TextColorR", 0);
	int nG				= GetProfileIntValue(nSliderNbr, "TextColorG", 0);
	int nB				= GetProfileIntValue(nSliderNbr, "TextColorB", 0);

	m_clrFontColor		= RGB(nR, nG, nB);

	nR					= GetProfileIntValue(nSliderNbr, "TextHighlightColorR", 255);
	nG					= GetProfileIntValue(nSliderNbr, "TextHighlightColorG", 255);
	nB					= GetProfileIntValue(nSliderNbr, "TextHighlightColorB", 255);

	m_clrFontHighliteColor = RGB(nR, nG, nB);

	// Do any conversions for the position
	CPoint pt = ConvertToScreenPosition(GetProfileIntValue(nSliderNbr, "PanelXPos", PANEL_DEFAULT),
										GetProfileIntValue(nSliderNbr, "PanelYPos", PANEL_DEFAULT));

	m_OrigRect.left		= pt.x;
	m_OrigRect.top		= pt.y;

	m_nFadeDelay		= GetProfileIntValue(nSliderNbr, "FadeDelay",	0);

	if(m_nFadeDelay > 32000)
		m_nFadeDelay = 32000;

	if(m_nFadeDelay < 0)
		m_nFadeDelay = 1;

	m_nFadeStep			= GetProfileIntValue(nSliderNbr, "FadeStep",	0);

	if(m_nFadeStep > 255)
		m_nFadeStep = 255;

	m_bUseToolTip		= GetProfileIntValue(nSliderNbr, "PanelToolTip", 1);

	cTemp = GetLSLnkImagePath(nSliderNbr,	"PanelMouseOverBitmap");

	if(cTemp.GetLength())
	{
		// Load the mouse over bitmap
		if(!LoadBitmap(m_OverBitmap, cTemp))
			ErrorLog("Can't load bitmap for PanelMouseOver!");

	}

	m_nTransparency = GetProfileIntValue(nSliderNbr, "Transparency", 0);

	m_nTransparency = 255 - m_nTransparency;

	return TRUE;
}

BOOL CSliderChild::AssembleVerticalBitmap()
{
	// Function will assemble the Child bitmap for verticle menu's
	CWindowDC dc(NULL);
	
	CRect		rect;
	CDC			memDC;
	CDC			workDC;

	CBitmap 	bitmap;
	CBitmap		bitmapTop;
	CBitmap		bitmapBottom;
	CBitmap		bitmapMiddle;

	DWORD		dwBitmapLength  = 0;
	DWORD		dwCurPos		= 0;

	int			nNumberOfLnkFiles	= 0;
	int			nIconIndex			= 0;
	
	CString		cIconPath	= _T("");
	CString		cDisplay	= _T("");
	CString		cErrorText  = _T("");

	HICON		hIcon		= NULL;

	BITMAP		bm;

	CStringArray strArray;

	// Get the Top
	if(LoadBitmap(bitmapTop, m_cTopBitmapPath))
	{
		bitmapTop.GetBitmap(&bm);
		dwBitmapLength += bm.bmHeight;
	}

	// Do the Bottom
	if(LoadBitmap(bitmapBottom, m_cBottomBitmapPath))
	{
		bitmapBottom.GetBitmap(&bm);

		dwBitmapLength += bm.bmHeight;
		m_nTopBitmapHeight = bm.bmHeight;

	}

	m_cExclusionList	= GetProfileStringValue(m_nSliderNumber, "ExclusionList");

	m_cExclusionList.MakeUpper();

	nNumberOfLnkFiles = GetLinkFilesInDirectory(m_cPath + m_WildCardSearch, strArray, m_cExclusionList);	

	if(nNumberOfLnkFiles == 0)
	{
		cErrorText.Format("Warning 0 files found in path - %s", m_cPath);
	}

	// Sort the Array
	SortStringArray(strArray, FALSE);

	// Do the middle, this has to be here
	if(LoadBitmap(bitmapMiddle, m_cMiddleBitmapPath))
	{
		bitmapMiddle.GetBitmap(&bm);
		dwBitmapLength += (bm.bmHeight * nNumberOfLnkFiles);
	}
	else
	{
		cErrorText.Format("Unable to load bitmap - %s", m_cMiddleBitmapPath);
		ErrorLog(cErrorText);

		return FALSE;
	}

	rect.left	= 0;
	rect.top	= 0;
	rect.right	= bm.bmWidth;
	rect.bottom	= dwBitmapLength;

	memDC.CreateCompatibleDC(&dc); 	
	workDC.CreateCompatibleDC(&dc); 	

	bitmap.CreateCompatibleBitmap(&dc, rect.Width(), rect.Height());	

	memDC.SelectObject(&bitmap);
	memDC.FillSolidRect(rect, RGB(255,0,255));

	// Assemble the Pieces

	// Top
	if(bitmapTop.m_hObject)
	{
		workDC.SelectObject(bitmapTop);

		bitmapTop.GetBitmap(&bm);
		memDC.BitBlt(0, dwCurPos, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);

		dwCurPos += bm.bmHeight;
	}

	workDC.SelectObject(bitmapMiddle);

	bitmapMiddle.GetBitmap(&bm);

	if(m_bPrintText && m_Font.m_hObject)
	{
		memDC.SelectObject(m_Font);
		memDC.SetBkMode(TRANSPARENT);
		memDC.SetTextColor(m_clrFontColor);
	}

	for(int i = 0; i < strArray.GetSize(); i++)
	{
		memDC.BitBlt(0, dwCurPos, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);

		// Create the Button class
		CSliderButton* pButton = new CSliderButton;

		if(NULL != pButton)
		{
			hIcon = NULL;

			// Set button rect
			pButton->m_Rect = CRect(0, dwCurPos, bm.bmWidth, dwCurPos + bm.bmHeight);

			// Store the link path
			pButton->m_LinkFile = m_cPath + strArray[i];

			if(m_bPrintIcon)
			{
				// Try to read right out of the Lnk file
				if(GetLinkIcon(pButton->m_LinkFile, cIconPath, nIconIndex))
				{	
					if(m_bUseSmallIcons)
						ExtractIconEx(cIconPath, nIconIndex, NULL, &hIcon, 1);
					else
						ExtractIconEx(cIconPath, nIconIndex, &hIcon, NULL, 1);
				}

				if(NULL == hIcon)
				{
					// Try to get the icon from Explorer
					SHFILEINFO shf;

					UINT nFlags = SHGFI_ICON;

					if(m_bUseSmallIcons)
						nFlags |= SHGFI_SMALLICON;
					else
						nFlags |= SHGFI_LARGEICON;

					SHGetFileInfo(pButton->m_LinkFile, NULL, &shf, sizeof(shf), nFlags);
					if (shf.hIcon != NULL)
						hIcon = shf.hIcon;
				}

				if(NULL != hIcon)
				{
					DrawIconEx(memDC.m_hDC, 0 + m_nIconXAdj, dwCurPos + m_nIconYAdj, hIcon, 
						m_bUseSmallIcons ? 16 : 32, 
						m_bUseSmallIcons ? 16 : 32, 0, NULL, DI_NORMAL);

					// Store the Icon
					pButton->m_hIcon = hIcon;

					// Store the size
					pButton->m_IconSize = m_bUseSmallIcons ? 16 : 32;

					// Store the position
					pButton->m_IconPos = CPoint(dwCurPos + m_nIconXAdj, 0 + m_nIconYAdj);
				}
			}

			// Do the text
			cDisplay= _T("");

			GetFileTitle(pButton->m_LinkFile, cDisplay.GetBuffer(25), 25);
			cDisplay.ReleaseBuffer();

			// Do the tooltip
			if(m_bUseToolTip)
				m_ToolTip.AddTool(this, cDisplay, pButton->m_Rect, i + 1);

			if(m_bPrintText)
			{
				memDC.TextOut(0 + m_nTextXAdj, dwCurPos + m_nTextYAdj, cDisplay);

				// Store the display
				pButton->m_DisplayName = cDisplay;

				// Store the text position
				pButton->m_TextPos = CPoint(0 + m_nTextXAdj, dwCurPos + m_nTextYAdj);

				// Store the Font info
				pButton->m_cFontText = m_cFontText;

				pButton->m_clrFontColor = m_clrFontColor;
				
				pButton->m_clrFontHighliteColor = m_clrFontHighliteColor;

				pButton->m_nFontHeight = m_nFontHeight;

			}

			// Add the button
			m_colSliderButtons.Add(pButton);
		}

		dwCurPos += bm.bmHeight;
	}

	// Do the Bottom
	if(bitmapBottom.m_hObject)
	{
		workDC.SelectObject(bitmapBottom);

		bitmapBottom.GetBitmap(&bm);
		memDC.BitBlt(0, dwCurPos, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);
	}

	m_Bitmap.Attach(bitmap.Detach());

	strArray.RemoveAll();

	return TRUE;
}

BOOL CSliderChild::AssembleHorizontalBitmap()
{
	// Function will assemble the Child bitmap for verticle menu's
	CWindowDC dc(NULL);
	
	CRect		rect;
	CDC			memDC;
	CDC			workDC;

	CBitmap 	bitmap;
	CBitmap		bitmapTop;
	CBitmap		bitmapBottom;
	CBitmap		bitmapMiddle;

	DWORD		dwBitmapWidth   = 0;
	DWORD		dwCurPos		= 0;

	int			nNumberOfLnkFiles	= 0;
	int			nIconIndex			= 0;

	CString		cIconPath	= _T("");
	CString		cDisplay	= _T("");
	CString		cErrorText  = _T("");

	CStringArray strArray;

	HICON		hIcon		= NULL;

	BITMAP		bm;

	nNumberOfLnkFiles = GetLinkFilesInDirectory(m_cPath + m_WildCardSearch, strArray, m_cExclusionList);

	if(nNumberOfLnkFiles == 0)
	{
		cErrorText.Format("Warning 0 files found in path - %s", m_cPath);
	}

	// Sort the Array
	SortStringArray(strArray, FALSE);

	if(LoadBitmap(bitmapTop, m_cTopBitmapPath))
	{
		bitmapTop.GetBitmap(&bm);
		dwBitmapWidth += bm.bmWidth;
	}

	if(LoadBitmap(bitmapBottom, m_cBottomBitmapPath))
	{
		bitmapBottom.GetBitmap(&bm);
		dwBitmapWidth += bm.bmWidth;
	}

	if(LoadBitmap(bitmapMiddle, m_cMiddleBitmapPath))
	{
		bitmapMiddle.GetBitmap(&bm);
		dwBitmapWidth += (bm.bmWidth * nNumberOfLnkFiles);
	}
	else
	{
		cErrorText.Format("Unable to load bitmap - %s", m_cMiddleBitmapPath);
		ErrorLog(cErrorText);

		return FALSE;
	}

	rect.left	= 0;
	rect.top	= 0;
	rect.right	= dwBitmapWidth;
	rect.bottom	= bm.bmHeight;

	memDC. CreateCompatibleDC(&dc); 	
	workDC.CreateCompatibleDC(&dc); 	

	bitmap.CreateCompatibleBitmap(&dc, rect.Width(), rect.Height());	

	memDC.SelectObject(&bitmap);
	memDC.FillSolidRect(rect, RGB(255,0,255));

	// Assemble the Pieces

	// To the top
	if(bitmapTop.m_hObject)
	{
		workDC.SelectObject(bitmapTop);

		bitmapTop.GetBitmap(&bm);
		memDC.BitBlt(dwCurPos, 0, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);

		dwCurPos += bm.bmWidth;
	}

	// do the middle stuff
	workDC.SelectObject(bitmapMiddle);

	bitmapMiddle.GetBitmap(&bm);

	if(m_bPrintText && m_Font.m_hObject)
	{
		memDC.SelectObject(m_Font);
		memDC.SetBkMode(TRANSPARENT);
		memDC.SetTextColor(m_clrFontColor);
	}

	for(int i = 0; i < strArray.GetSize(); i++)
	{
		// Copy the background in
		memDC.BitBlt(dwCurPos, 0, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);

		// Create the Button class
		CSliderButton* pButton = new CSliderButton;

		if(NULL != pButton)
		{
			// Set button rect
			pButton->m_Rect = CRect(dwCurPos, 0,bm.bmWidth + dwCurPos, bm.bmHeight);

			// Store the link path
			pButton->m_LinkFile = m_cPath + strArray[i];

			if(m_bPrintIcon)
			{
				// Try to read right out of the Lnk file
				if(GetLinkIcon(pButton->m_LinkFile, cIconPath, nIconIndex))
				{	
					if(m_bUseSmallIcons)
						ExtractIconEx(cIconPath, nIconIndex, NULL, &hIcon, 1);
					else
						ExtractIconEx(cIconPath, nIconIndex, &hIcon, NULL, 1);
				}

				if(NULL == hIcon)
				{
					// Try to get the icon from Explorer
					SHFILEINFO shf;

					UINT nFlags = SHGFI_ICON;

					if(m_bUseSmallIcons)
						nFlags |= SHGFI_SMALLICON;
					else
						nFlags |= SHGFI_LARGEICON;

					SHGetFileInfo(pButton->m_LinkFile, NULL, &shf, sizeof(shf), nFlags);
					if (shf.hIcon != NULL)
						hIcon = shf.hIcon;
				}
				
				if(NULL != hIcon)
				{
					DrawIconEx(memDC.m_hDC, dwCurPos + m_nIconXAdj, 0 + m_nIconYAdj, hIcon, 
						m_bUseSmallIcons ? 16 : 32, 
						m_bUseSmallIcons ? 16 : 32, 0, NULL, DI_NORMAL);

					// Store the Icon
					pButton->m_hIcon = hIcon;

					// Store the size
					pButton->m_IconSize = m_bUseSmallIcons ? 16 : 32;

					// Store the position
					pButton->m_IconPos = CPoint(dwCurPos + m_nIconXAdj, 0 + m_nIconYAdj);
				}
			}

			// Do the text
			cDisplay= _T("");

			GetFileTitle(pButton->m_LinkFile, cDisplay.GetBuffer(25), 25);
			cDisplay.ReleaseBuffer();

			// Do the tooltip
			if(m_bUseToolTip)
				m_ToolTip.AddTool(this, cDisplay, pButton->m_Rect, i + 1);

			// Do the text
			if(m_bPrintText)
			{
				memDC.ExtTextOut(dwCurPos + m_nTextXAdj, 0 + m_nTextYAdj, ETO_CLIPPED,
					pButton->m_Rect, cDisplay, NULL);

				// Store the display
				pButton->m_DisplayName = cDisplay;

				// Store the text position
				pButton->m_TextPos = CPoint(dwCurPos + m_nTextXAdj, 0 + m_nTextYAdj);

				// Store the Font info
				pButton->m_cFontText = m_cFontText;

				pButton->m_clrFontColor = m_clrFontColor;

				pButton->m_clrFontHighliteColor = m_clrFontHighliteColor;

				pButton->m_nFontHeight = m_nFontHeight;
			}

			// Add the button
			m_colSliderButtons.Add(pButton);

		}

		dwCurPos += bm.bmWidth;
	}

	// Do the bottom
	if(bitmapBottom.m_hObject)
	{
		workDC.SelectObject(bitmapBottom);

		bitmapBottom.GetBitmap(&bm);
		memDC.BitBlt(dwCurPos, 0, bm.bmWidth, bm.bmHeight, &workDC, 0, 0, SRCCOPY);
	}

	m_Bitmap.Attach(bitmap.Detach());

	strArray.RemoveAll();

	return TRUE;
}

LRESULT CSliderChild::OnMouseLeave(WPARAM wParam, LPARAM lParam)
{
	//Indicate that the mouse is not over us
	m_bMouseTracking = FALSE;

	m_nCurBtn = -1;
			
	if(m_bUseToolTip)
	{
		// Get rid of any ToolTip
		MSG TooltipMessage;

		TooltipMessage.hwnd		= m_hWnd;
		TooltipMessage.message	= WM_MOUSEMOVE;
		TooltipMessage.wParam	= 0;
		TooltipMessage.lParam	= MAKELPARAM(-1, -1);
		TooltipMessage.pt		= CPoint(-1,-1);

		m_ToolTip.RelayEvent(&TooltipMessage);
	}

	Invalidate();
	UpdateWindow();

	return TRUE;
}

void CSliderChild::OnMouseMove(UINT nFlags, CPoint point) 
{
	CWnd::OnMouseMove(nFlags, point);

	if(m_bUseToolTip)
	{
		// The RelayEvent needs screen coordinates
		CPoint ScreenPt = point;
		ClientToScreen(&ScreenPt);

		MSG TooltipMessage;

		TooltipMessage.hwnd		= m_hWnd;
		TooltipMessage.message	= WM_MOUSEMOVE;
		TooltipMessage.wParam	= nFlags;
		TooltipMessage.lParam	= MAKELPARAM(ScreenPt.x, ScreenPt.y);
		TooltipMessage.pt		= ScreenPt;

		m_ToolTip.RelayEvent(&TooltipMessage);
	}

	CSliderButton* pButton = NULL;

	// See if any field was hit and tell them to draw
	for(int i = 0; i < m_colSliderButtons.GetSize(); i++)
	{
		pButton = (CSliderButton*)m_colSliderButtons[i];

		if(NULL != pButton)
		{
			if(m_nCurBtn != i && pButton->HitTest(point))
			{
				m_nOldBtn = m_nCurBtn;
				m_nCurBtn = i;

				// Play the sound
				if(m_MouseOverSound.GetLength())
					PlaySound(m_MouseOverSound, NULL, SND_ASYNC | SND_FILENAME);

				if(!m_bMouseTracking)
				{
					m_bMouseTracking = TRUE;

					// set the timer
					SetTimer(MOUSE_OVER_TIMER_ID, 50, NULL);
				}

				Invalidate();
				UpdateWindow();

				break;
			}			
		}
	}	

}

void CSliderChild::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// See if any field was hit and execute them
	for(int i = 0; i < m_colSliderButtons.GetSize(); i++)
	{
		CSliderButton* pButton = (CSliderButton*)m_colSliderButtons[i];

		if(NULL != pButton)
		{
			if(pButton->HitTest(point))
			{
				// Play the sound
				if(m_PressSound.GetLength())
					PlaySound(m_PressSound, NULL, SND_ASYNC | SND_FILENAME);

				pButton->Execute();

				// Roll it up
				Sleep(50);
				Animate(0);
				break;
			}
		}
	}
	
	CWnd::OnLButtonDown(nFlags, point);
}

void CSliderChild::OnTimer(UINT nIDEvent) 
{
	if(MOUSE_OVER_TIMER_ID == nIDEvent)
	{
		//Check our mouse pos and redraw if necessary
		POINT curPT;
		GetCursorPos(&curPT);
					 
		RECT rect;
		GetWindowRect(&rect);
		
		if(!PtInRect(&rect, curPT) )
		{
			KillTimer(MOUSE_OVER_TIMER_ID);
			
			m_nOldBtn = m_nCurBtn;
			m_nCurBtn = 0;			

			m_bMouseTracking = FALSE;
			
			if(m_bUseToolTip)
			{
				// Get rid of any ToolTip
				MSG TooltipMessage;

				TooltipMessage.hwnd		= m_hWnd;
				TooltipMessage.message	= WM_MOUSEMOVE;
				TooltipMessage.wParam	= 0;
				TooltipMessage.lParam	= MAKELPARAM(curPT.x, curPT.y);
				TooltipMessage.pt		= curPT;

				m_ToolTip.RelayEvent(&TooltipMessage);
			}

			Invalidate();
		}	
	}
	
	CWnd::OnTimer(nIDEvent);
}

void CSliderChild::OnKillFocus(CWnd* pNewWnd) 
{
	// Roll Us up
	if(m_bExpanded)
	{
		if(NULL != pNewWnd && (pNewWnd->m_hWnd == GetParent()->m_hWnd))
			return;

		SetWindowPos(NULL, m_OrigRect.left, m_OrigRect.top, 0, 0, SWP_NOZORDER);
		Invalidate();

		m_bExpanded = FALSE;		
		GetParent()->SendMessage(WM_CHILD_CONTRACTED);		
	}

	CWnd::OnKillFocus(pNewWnd);
}

void CSliderChild::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CDC memDC;
	CDC BufferDC;

	CDib dibBack;
	CDib dibOurBitmap;

	CBitmap	BufferBitmap;

	CRect rect;

	GetWindowRect(rect);

	BITMAP bm;

	m_Bitmap.GetBitmap(&bm);
	m_hRgn = BitmapToRegion(m_Bitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	memDC.CreateCompatibleDC(&dc);
	memDC.SelectObject(&m_Bitmap);

	BufferDC.CreateCompatibleDC(&dc);
	BufferBitmap.CreateCompatibleBitmap(&dc, bm.bmWidth, bm.bmHeight);
	BufferDC.SelectObject(BufferBitmap);

	BufferDC.BitBlt(0, 0, bm.bmWidth, bm.bmHeight, &memDC, 0, 0, SRCCOPY);

	// Tell the buttons to draw itself
	CSliderButton* pButton = NULL;

	if(m_colSliderButtons.GetSize() && m_nCurBtn <= m_colSliderButtons.GetSize())
		pButton = (CSliderButton*)m_colSliderButtons[m_nCurBtn];

	if(pButton)
	{
		// See if we need to account for a mouse over bitmap
		if(m_OverBitmap.m_hObject)
		{
			BITMAP bm;
			memDC.SelectObject(&m_OverBitmap);

			m_OverBitmap.GetBitmap(&bm);

			// Draw the mouse over
			BufferDC.BitBlt(0, m_nCurBtn * bm.bmHeight + m_nTopBitmapHeight, bm.bmWidth, bm.bmHeight, &memDC, 0, 0, SRCCOPY);

			pButton->Draw(&BufferDC, TRUE);
		}
		else
			pButton->Draw(&BufferDC, TRUE);
	}

	dibOurBitmap.PasteDC(&BufferDC, 0, 0, bm.bmWidth, bm.bmHeight);

	dibBack.Paste(&m_dibScreen);
	dibBack.Blend(&dibOurBitmap, m_nTransparency);

	dibBack.Render(&dc, bm.bmWidth, bm.bmHeight);	

}

void CSliderChild::ResetPanel()
{
	// Called when m_bDynamic is true.
	if(m_Bitmap.m_hObject)
		m_Bitmap.DeleteObject();

	// Remove all the buttons
	for(int i = m_colSliderButtons.GetUpperBound(); i >= 0; i--)
	{
		delete m_colSliderButtons[i];
	}

	m_colSliderButtons.RemoveAll();

	// ReCreate it
	if(1 == m_nOrientation)
		AssembleVerticalBitmap();
	else
		AssembleHorizontalBitmap();
}

void CSliderChild::Animate(int nStep)
{
	// Called to expand or close the window


	if(m_bExpanded)
	{
		if(m_nSlideDirection == 8)
			FoldOut();

		m_bExpanded = FALSE;
		SetWindowPos(NULL, m_OrigRect.left, m_OrigRect.top, 0, 0, SWP_NOZORDER);

		GetParent()->SendMessage(WM_CHILD_CONTRACTED);
		return;
	}
	else
	{
		// Get the Background Buffer
		CRect rect;
		GetWindowRect(rect);
		CDC* pScreenDC = GetDesktopWindow()->GetDC();
		
		BITMAP bm;
		m_Bitmap.GetBitmap(&bm);

		m_dibScreen.PasteDC(pScreenDC, m_OrigRect.left, m_OrigRect.top, bm.bmWidth, bm.bmHeight);
		ReleaseDC(pScreenDC);

		if(m_bDynamic)
			ResetPanel();

		m_nCurBtn = 0;

		switch(m_nSlideDirection)
		{
			case 1:			// Down
				SlideDown(m_nStep);
				break;

			case 2:			// Up
				SlideUp(m_nStep);
				break;

			case 3:			// Left
				SlideLeft(m_nStep);
				break;

			case 4:			// Right
				SlideRight(m_nStep);
				break;

			case 5:			// Right and Down
				SlideRightAndDown(m_nStep);
				break;

			case 6:
				SlideRightAndUp(m_nStep);
				break;

			case 7:
				FadeIn();
				break;

			case 8:
				FoldIn();
				break;

			case 9:
				FoldOut();
				break;

			default:
				SlideDown(m_nStep);
				break;
		}

		m_bExpanded = TRUE;
	}
}

BOOL CSliderChild::IsExpanded()
{
	// Called to see if we are expanded
	return m_bExpanded;
}

void CSliderChild::ExpandFullDown()
{
	// Function will make the Window fully expanded downward
	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	SetWindowPos(NULL, 0, 0, bm.bmWidth, bm.bmHeight, SWP_NOZORDER | SWP_NOMOVE);
	
	Invalidate();
	UpdateWindow();
}

void CSliderChild::ExpandFullUp()
{
	// Function will make the Window fully expanded upward
	CRect rect;

	GetWindowRect(rect);

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	SetWindowPos(NULL, rect.left, rect.bottom - bm.bmHeight, bm.bmWidth, bm.bmHeight, SWP_NOZORDER);
	
	Invalidate();
	UpdateWindow();
}


void CSliderChild::SlideRightAndDown(int nStep)
{
	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	double nModifier = 1;
	double nWidth	= bm.bmWidth;
	double nHeight	= bm.bmHeight;

	if(nHeight >= nWidth)
	{
		// Determine the modifier
		nModifier = nWidth / nHeight;

		for(int i = nStep; i <= bm.bmHeight; i += nStep)
		{
			SetWindowPos(NULL, 0, 0, (int)(i * nModifier), i, SWP_NOZORDER | SWP_NOMOVE);
			Invalidate();
			UpdateWindow();
		}
	}
	else
	{
		// Determine the modifier
		nModifier = nHeight / nWidth;

		for(int i = nStep; i <= bm.bmWidth; i += nStep)
		{
			SetWindowPos(NULL, 0, 0, i, (int)(i * nModifier), SWP_NOZORDER | SWP_NOMOVE);
			Invalidate();
			UpdateWindow();
		}
	}

	// Fully expand it to make sure
	ExpandFullDown();
}

void CSliderChild::SlideDown(int nStep)
{
	CRect rect;

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);


	for(int i = nStep; i <= bm.bmHeight; i += nStep)
	{
		SetWindowPos(NULL, 0, 0, bm.bmWidth, i, SWP_NOZORDER | SWP_NOMOVE);
		Invalidate();
		UpdateWindow();
	}

	ExpandFullDown();
}

void CSliderChild::SlideUp(int nStep)
{
	CRect rect;

	GetWindowRect(rect);

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	for(int i = 0; i <= bm.bmHeight; i += nStep)
	{
		SetWindowPos(NULL, rect.left, rect.top - i, bm.bmWidth, bm.bmHeight - (bm.bmHeight - i), SWP_NOZORDER);
		Invalidate();
		UpdateWindow();

		Sleep(10);
	}

	ExpandFullUp();
}

void CSliderChild::SlideRight(int nStep)
{
	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	for(int i = nStep; i <= bm.bmWidth; i += nStep)
	{
		SetWindowPos(NULL, 0, 0, i, bm.bmHeight, SWP_NOZORDER | SWP_NOMOVE);
		Invalidate();
		UpdateWindow();
	}

	ExpandFullDown();
}


void CSliderChild::SlideLeft(int nStep)
{
	CRect rect;
	GetWindowRect(rect);

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	for(int i = nStep; i <= bm.bmWidth; i += nStep)
	{
		SetWindowPos(NULL, rect.left - i, rect.top, bm.bmWidth - (bm.bmWidth - i), bm.bmHeight, SWP_NOZORDER);
		Invalidate();
		UpdateWindow();
	}

	ExpandFullDown();

}

void CSliderChild::SlideRightAndUp(int nStep)
{
	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	CRect rect;
	CRect rectWidth;

	GetWindowRect(rect);

	double nModifier = 1;
	double nWidth	 = bm.bmWidth;
	double nHeight	 = bm.bmHeight;

	if(nHeight >= nWidth)
	{
		// Determine the modifier
		nModifier = nWidth / nHeight;

		for(int i = nStep; i <= bm.bmHeight; i += nStep)
		{			
			GetWindowRect(rectWidth);

			SetWindowPos(NULL, rect.left, (int)(rect.top - (i * nModifier)), (int)(i * nModifier), bm.bmHeight - (bm.bmHeight - i), SWP_NOZORDER);
			Invalidate();
			UpdateWindow();

		}
	}

	ExpandFullUp();
}

void CSliderChild::FadeIn()
{
	CDib dibScreen;
	CDib dibOurBitmap;
	CDib dibBack;

	CRect rect;

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	// Make the Window full
	SetWindowPos(NULL, 0, 0, bm.bmWidth, bm.bmHeight, SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);
	
	GetWindowRect(rect);

	CDC* dc = GetDC();

	int nSteps = 0;

	m_hRgn = BitmapToRegion(m_Bitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	// Get the Background
	CDC* pScreenDC = GetDesktopWindow()->GetDC();
	dibScreen.PasteDC(pScreenDC, rect.left, rect.top, rect.Width(), rect.Height());

	dibOurBitmap.Create(bm.bmWidth, bm.bmHeight);
	dibOurBitmap.SetBitmap(dc, &m_Bitmap);

	dibBack.Create(bm.bmWidth, bm.bmHeight );

	if(m_nFadeStep != 0)
		nSteps = 255 / m_nFadeStep;

	for(int i= 0; i < 255; i += nSteps)
	{
		// Shade it
		dibBack.Paste(&dibScreen);
		dibBack.Blend(&dibOurBitmap, i);
		dibBack.Render(dc, bm.bmWidth, bm.bmHeight);

		if(m_nFadeDelay != 0)
			Sleep(m_nFadeDelay);
	}

	ReleaseDC(pScreenDC);
	ReleaseDC(dc);

	Invalidate();
}

void CSliderChild::FoldIn()
{
	CDIBSectionLite MenuDIB;
	CDib dibBack;
	CDib dibOurBitmap;
	CDib dibScreen;

	CBitmap memBitmap;

	CRect rect;
	int i;

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	// Make the Window full
	SetWindowPos(NULL, 0, 0, bm.bmWidth, bm.bmHeight, SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);
	
	GetWindowRect(rect);

	CDC* dc = GetDC();
	CDC  memDC;

	memDC.CreateCompatibleDC(dc);
	memBitmap.CreateCompatibleBitmap(dc, bm.bmWidth, bm.bmHeight);

	memDC.SelectObject(&memBitmap);

	int nSteps = 0;

	m_hRgn = BitmapToRegion(m_Bitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	// Get the Background
	m_dibScreen.Render(&memDC, bm.bmWidth, bm.bmHeight);
	
	MenuDIB.SetBitmap(m_Bitmap);

	CPoint pt(0, 0);
	CSize  size(bm.bmWidth, 10);

	dibBack.Create(bm.bmWidth, bm.bmHeight);

	for(i = 1; i < (bm.bmHeight - 15); i += m_nFadeStep)
	{
		dibOurBitmap.Create(bm.bmWidth, i);

		MenuDIB.Stretch(&memDC, pt, size);

		dibOurBitmap.PasteDC(&memDC, 0, 0, bm.bmWidth, bm.bmHeight);
		
		dibBack.Paste(&m_dibScreen);
		dibBack.Blend(&dibOurBitmap, m_nTransparency);

		dibBack.Render(dc, bm.bmWidth, bm.bmHeight);

		size.cy = i;
		Sleep(m_nFadeDelay);
	}

	// Make sure we glide all the way done
	for(; i <= bm.bmHeight; i += 1)
	{
		dibOurBitmap.Create(bm.bmWidth, bm.bmHeight);

		MenuDIB.Stretch(&memDC, pt, size);

		dibOurBitmap.PasteDC(&memDC, 0, 0, bm.bmWidth, bm.bmHeight);
		
		dibBack.Paste(&m_dibScreen);
		dibBack.Blend(&dibOurBitmap, m_nTransparency);
		dibBack.Render(dc, bm.bmWidth, bm.bmHeight);

		size.cy = i;
		Sleep(m_nFadeDelay);
	}

	ReleaseDC(dc);
	ExpandFullDown();
	Invalidate();
}

void CSliderChild::FoldOut()
{
	CDIBSectionLite MenuDIB;
	CDib dibBack;
	CDib dibOurBitmap;

	CBitmap memBitmap;

	CRect rect;
	int i;

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	// Make the Window full
	SetWindowPos(NULL, 0, 0, bm.bmWidth, bm.bmHeight, SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);
	
	GetWindowRect(rect);

	CDC* dc = GetDC();
	CDC  memDC;

	memDC.CreateCompatibleDC(dc);
	memBitmap.CreateCompatibleBitmap(dc, bm.bmWidth, bm.bmHeight);

	memDC.SelectObject(&memBitmap);

	int nSteps = 0;

	m_hRgn = BitmapToRegion(m_Bitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	// Get the Background
	m_dibScreen.Render(&memDC, bm.bmWidth, bm.bmHeight);
	MenuDIB.SetBitmap(m_Bitmap);

	CPoint pt(0, 0);
	CSize  size(bm.bmWidth, bm.bmHeight);	

	for(i = bm.bmHeight; i > 15; i -= m_nFadeStep)
	{
		dibOurBitmap.Create(bm.bmWidth, i);

		m_dibScreen.Render(&memDC, bm.bmWidth, bm.bmHeight);
		MenuDIB.Stretch(&memDC, pt, size);

		dibOurBitmap.PasteDC(&memDC, 0, 0, bm.bmWidth, bm.bmHeight);
		
		dibBack.Paste(&m_dibScreen);
		dibBack.Blend(&dibOurBitmap, m_nTransparency);
		dibBack.Render(dc, bm.bmWidth, bm.bmHeight);

		size.cy = i;
		Sleep(m_nFadeDelay);
	}

	// Make sure we glide all the way done
	for(; i > 0; i -= 1)
	{
		dibOurBitmap.Create(bm.bmWidth, bm.bmHeight);

		m_dibScreen.Render(&memDC, bm.bmWidth, bm.bmHeight);

		MenuDIB.Stretch(&memDC, pt, size);

		dibOurBitmap.PasteDC(&memDC, 0, 0, bm.bmWidth, bm.bmHeight);
		
		dibBack.Paste(&m_dibScreen);
		dibBack.Blend(&dibOurBitmap, m_nTransparency);
		dibBack.Render(dc, bm.bmWidth, bm.bmHeight);

		size.cy = i;
		Sleep(m_nFadeDelay);
	}

	m_bExpanded = FALSE;
	ReleaseDC(dc);	
}

void CSliderChild::GenerateBackBuffer(CDC* pDestDC)
{
	// Called to take a snapshot of what the background 
	// should look like before any text is drawn on the destination

	if(NULL == pDestDC)
		return;

	CRect rect;
	GetWindowRect(rect);

	CWnd* pDesktop = CWnd::GetDesktopWindow();
	
	CDC* DesktopDC = pDesktop->GetDC();

	CBitmap FrameBM;
	FrameBM.CreateCompatibleBitmap(DesktopDC, rect.Width(), rect.Height());

	BITMAP bm;
	m_Bitmap.GetBitmap(&bm);

	pDestDC->SelectObject(&FrameBM);
	pDestDC->BitBlt(0, 0, bm.bmWidth, bm.bmHeight,
					DesktopDC, rect.left, rect.top, SRCCOPY);

	pDesktop->ReleaseDC(DesktopDC);
}

BOOL CSliderChild::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	// TODO: Add your message handler code here and/or call default
	
	return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

void CSliderChild::Execute(int nButtonNbr)
{
	// Called to execute one of our buttons
	if(nButtonNbr <= 0 || nButtonNbr > m_colSliderButtons.GetSize())
		// Invalid, bail
		return;

	((CSliderButton*)m_colSliderButtons[nButtonNbr - 1])->Execute();
}

void CSliderChild::Execute(CString cBtnDesctription)
{
	// Called to execute one of our buttons by description
	CSliderButton* pButton = NULL;

	for(int i = m_colSliderButtons.GetUpperBound(); i >= 0; i--)
	{
		pButton = (CSliderButton*)m_colSliderButtons[i];

		if(pButton)
		{
			if(0 == cBtnDesctription.CompareNoCase(pButton->m_DisplayName))
			{
				// Run it
				pButton->Execute();

				break;
			}
		}
	}
}

void CSliderChild::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch(nChar)
	{
		case VK_DOWN:
			if(m_nCurBtn == (m_colSliderButtons.GetSize() - 1))
			{
				m_nOldBtn = m_nCurBtn;
				m_nCurBtn = 0;
			}
			else
			{
				m_nOldBtn = m_nCurBtn;
				m_nCurBtn++;
			}

			Invalidate();
			break;

		case VK_UP:
			if(m_nCurBtn == 0)
			{
				m_nOldBtn = m_nCurBtn;
				m_nCurBtn = m_colSliderButtons.GetSize() - 1;
			}
			else
			{
				m_nOldBtn = m_nCurBtn;
				m_nCurBtn--;
			}

			Invalidate();
			break;

		case VK_RETURN:
		case VK_SPACE:
		{
			CSliderButton* pButton = (CSliderButton*)m_colSliderButtons[m_nCurBtn];

			if(NULL != pButton)
			{
				pButton->Execute();

				// Roll it up
				Sleep(50);
				Animate(0);
			}
		}
			break;

		default:
			break;
	}
		
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CSliderChild::OnParentMove(HWND hParent)
{
	// Called from the parent when they move. we may have to move with them

	CalcPosition(hParent);

	SetWindowPos(NULL, m_OrigRect.left, m_OrigRect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}
